"""
A collection of constants put here because they should be either
* easily accessible on the camera by a developer or
* globally consistent and available.

Since these are to be read only and have no dependencies they will be available
immediately.

Note that these are not suitable for overriding camera configuration because
they will be replaced when software is updated.

Note that these should not be written to at runtime because that would make it
very hard to trace where values come from and where they are consumed.
Particularly problematic is the case where a global variable is changed more
than once such that different modules may consume different values at different
times.
"""


class _Constant(object):
    def __init__(self, value):
        self._value = value

    def __get__(self, instance, owner):
        return self._value

    def __set__(self, instance, value):
        raise TypeError("Constant objects do not support assignment")

    def __delete__(self, instance):
        raise TypeError("Constant objects do not support deletions")


def Constant(value):
    # Helper to trick the type system
    return _Constant(value)


class _Constants(object):
    # Start the app with a friendly watchdog that will not kill the process. Useful
    # when running the application with breakpoints.
    friendly_watchdog = Constant(False)  # type: bool


constants = _Constants()
